@@ -4,7 +4,7 @@
 
 "use strict";
 
-this.ContentSearchUIController = (function() {
+this.ContentSearchUIController = (function () {
   const MAX_DISPLAYED_SUGGESTIONS = 6;
   const SUGGESTION_ID_PREFIX = "searchSuggestion";
   const ONE_OFF_ID_PREFIX = "oneOff";
@@ -47,7 +47,7 @@ this.ContentSearchUIController = (function() {
     this._idPrefix = idPrefix;
     this._healthReportKey = healthReportKey;
     this._searchPurpose = searchPurpose;
-    this._isPrivateWindow = false;
+    this._isPrivateEngine = false;
 
     let tableID = idPrefix + "searchSuggestionTable";
     this.input.autocomplete = "off";
@@ -587,10 +587,7 @@ this.ContentSearchUIController = (function() {
 
       // Add the suggestions to the table.
       let searchWords = new Set(
-        suggestions.searchString
-          .trim()
-          .toLowerCase()
-          .split(/\s+/)
+        suggestions.searchString.trim().toLowerCase().split(/\s+/)
       );
       for (let i = 0; i < MAX_DISPLAYED_SUGGESTIONS; i++) {
         let type, idx;
@@ -620,10 +617,6 @@ this.ContentSearchUIController = (function() {
         );
         this._table.hidden = false;
         this.input.setAttribute("aria-expanded", "true");
-        this._originalDefaultEngine = {
-          name: this.defaultEngine.name,
-          icon: this.defaultEngine.icon,
-        };
       }
     },
 
@@ -636,13 +629,13 @@ this.ContentSearchUIController = (function() {
     _onMsgState(state) {
       // Not all state messages broadcast the windows' privateness info.
       if ("isPrivateWindow" in state) {
-        this._isPrivateWindow = state.isPrivateWindow;
+        this._isPrivateEngine = state.isPrivateEngine;
       }
 
       this.engines = state.engines;
 
       let currentEngine = state.currentEngine;
-      if (this._isPrivateWindow) {
+      if (this._isPrivateEngine) {
         currentEngine = state.currentPrivateEngine;
       }
 
@@ -662,7 +655,7 @@ this.ContentSearchUIController = (function() {
     },
 
     _onMsgCurrentEngine(engine) {
-      if (this._isPrivateWindow) {
+      if (this._isPrivateEngine) {
         return;
       }
       this.defaultEngine = engine;
@@ -670,7 +663,7 @@ this.ContentSearchUIController = (function() {
     },
 
     _onMsgCurrentPrivateEngine(engine) {
-      if (!this._isPrivateWindow) {
+      if (!this._isPrivateEngine) {
         return;
       }
       this.defaultEngine = engine;
@@ -681,9 +674,8 @@ this.ContentSearchUIController = (function() {
       this._strings = strings;
       this._updateDefaultEngineHeader();
       this._updateSearchWithHeader();
-      document.getElementById(
-        "contentSearchSettingsButton"
-      ).textContent = this._strings.searchSettings;
+      document.getElementById("contentSearchSettingsButton").textContent =
+        this._strings.searchSettings;
     },
 
     _updateDefaultEngineIcon() {
@@ -761,10 +753,7 @@ this.ContentSearchUIController = (function() {
       entry.id = this._idPrefix + SUGGESTION_ID_PREFIX + currentRow;
       entry.setAttribute("aria-selected", "false");
 
-      let suggestionWords = suggestionStr
-        .trim()
-        .toLowerCase()
-        .split(/\s+/);
+      let suggestionWords = suggestionStr.trim().toLowerCase().split(/\s+/);
       for (let i = 0; i < suggestionWords.length; i++) {
         let word = suggestionWords[i];
         let wordSpan = document.createElementNS(HTML_NS, "span");
@@ -986,7 +975,7 @@ this.ContentSearchUIController = (function() {
         img.setAttribute("src", uri);
         img.addEventListener(
           "load",
-          function() {
+          function () {
             URL.revokeObjectURL(uri);
           },
           { once: true }
