@@ -5,26 +5,33 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "VRManagerChild.h"
+
+#include "VRLayerChild.h"
 #include "VRManagerParent.h"
 #include "VRThread.h"
 #include "VRDisplayClient.h"
 #include "nsGlobalWindow.h"
+#include "mozilla/ProfilerMarkers.h"
 #include "mozilla/StaticPtr.h"
 #include "mozilla/layers/CompositorThread.h"  // for CompositorThread
 #include "mozilla/dom/Navigator.h"
 #include "mozilla/dom/VREventObserver.h"
+#include "mozilla/dom/WebXRBinding.h"
 #include "mozilla/dom/WindowBinding.h"  // for FrameRequestCallback
+#include "mozilla/dom/XRSystem.h"
+#include "mozilla/dom/XRFrame.h"
 #include "mozilla/dom/ContentChild.h"
 #include "nsContentUtils.h"
 #include "mozilla/dom/GamepadManager.h"
-#include "mozilla/dom/VRServiceTest.h"
+#include "mozilla/ipc/Endpoint.h"
 #include "mozilla/layers/SyncObject.h"
+#include "mozilla/layers/TextureForwarder.h"
 
 using namespace mozilla::dom;
 
 namespace {
-const nsTArray<RefPtr<dom::VREventObserver>>::index_type kNoIndex =
-    nsTArray<RefPtr<dom::VREventObserver>>::NoIndex;
+const nsTArray<RefPtr<mozilla::gfx::VRManagerEventObserver>>::index_type
+    kNoIndex = nsTArray<RefPtr<mozilla::gfx::VRManagerEventObserver>>::NoIndex;
 }  // namespace
 
 namespace mozilla {
@@ -33,16 +40,16 @@ namespace gfx {
 static StaticRefPtr<VRManagerChild> sVRManagerChildSingleton;
 static StaticRefPtr<VRManagerParent> sVRManagerParentSingleton;
 
+static TimeStamp sMostRecentFrameEnd;
+static TimeDuration sAverageFrameInterval;
+
 void ReleaseVRManagerParentSingleton() { sVRManagerParentSingleton = nullptr; }
 
 VRManagerChild::VRManagerChild()
-    : mDisplaysInitialized(false),
-      mMessageLoop(MessageLoop::current()),
+    : mRuntimeCapabilities(VRDisplayCapabilityFlags::Cap_None),
       mFrameRequestCallbackCounter(0),
-      mBackend(layers::LayersBackend::LAYERS_NONE),
-      mPromiseID(0),
-      mVRMockDisplay(nullptr),
-      mLastControllerState{} {
+      mWaitingForEnumeration(false),
+      mBackend(layers::LayersBackend::LAYERS_NONE) {
   MOZ_ASSERT(NS_IsMainThread());
 
   mStartTimeStamp = TimeStamp::Now();
@@ -55,9 +62,6 @@ void VRManagerChild::IdentifyTextureHost(
     const TextureFactoryIdentifier& aIdentifier) {
   if (sVRManagerChildSingleton) {
     sVRManagerChildSingleton->mBackend = aIdentifier.mParentBackend;
-    sVRManagerChildSingleton->mSyncObject =
-        layers::SyncObjectClient::CreateSyncObjectClient(
-            aIdentifier.mSyncHandle);
   }
 }
 
@@ -74,10 +78,35 @@ VRManagerChild* VRManagerChild::Get() {
 /* static */
 bool VRManagerChild::IsCreated() { return !!sVRManagerChildSingleton; }
 
+/* static */
+bool VRManagerChild::IsPresenting() {
+  if (!VRManagerChild::IsCreated()) {
+    return false;
+  }
+
+  nsTArray<RefPtr<VRDisplayClient>> displays;
+  sVRManagerChildSingleton->GetVRDisplays(displays);
+
+  bool result = false;
+  for (auto& display : displays) {
+    result |= display->IsPresenting();
+  }
+  return result;
+}
+
+TimeStamp VRManagerChild::GetIdleDeadlineHint(TimeStamp aDefault) {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!VRManagerChild::IsCreated() || sMostRecentFrameEnd.IsNull()) {
+    return aDefault;
+  }
+
+  TimeStamp idleEnd = sMostRecentFrameEnd + sAverageFrameInterval;
+  return idleEnd < aDefault ? idleEnd : aDefault;
+}
+
 /* static */
 bool VRManagerChild::InitForContent(Endpoint<PVRManagerChild>&& aEndpoint) {
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(!sVRManagerChildSingleton);
 
   RefPtr<VRManagerChild> child(new VRManagerChild());
   if (!aEndpoint.Bind(child)) {
@@ -87,15 +116,6 @@ bool VRManagerChild::InitForContent(Endpoint<PVRManagerChild>&& aEndpoint) {
   return true;
 }
 
-/* static */
-bool VRManagerChild::ReinitForContent(Endpoint<PVRManagerChild>&& aEndpoint) {
-  MOZ_ASSERT(NS_IsMainThread());
-
-  ShutDown();
-
-  return InitForContent(std::move(aEndpoint));
-}
-
 /*static*/
 void VRManagerChild::InitSameProcess() {
   MOZ_ASSERT(NS_IsMainThread());
@@ -103,8 +123,7 @@ void VRManagerChild::InitSameProcess() {
 
   sVRManagerChildSingleton = new VRManagerChild();
   sVRManagerParentSingleton = VRManagerParent::CreateSameProcess();
-  sVRManagerChildSingleton->Open(sVRManagerParentSingleton->GetIPCChannel(),
-                                 CompositorThreadHolder::Loop(),
+  sVRManagerChildSingleton->Open(sVRManagerParentSingleton, CompositorThread(),
                                  mozilla::ipc::ChildSide);
 }
 
@@ -122,27 +141,19 @@ void VRManagerChild::InitWithGPUProcess(Endpoint<PVRManagerChild>&& aEndpoint) {
 /*static*/
 void VRManagerChild::ShutDown() {
   MOZ_ASSERT(NS_IsMainThread());
-  if (sVRManagerChildSingleton) {
-    sVRManagerChildSingleton->Destroy();
+  if (!sVRManagerChildSingleton) {
+    return;
+  }
+  sVRManagerChildSingleton->Close();
+  sVRManagerChildSingleton = nullptr;
+}
+
+void VRManagerChild::ActorDestroy(ActorDestroyReason aReason) {
+  if (sVRManagerChildSingleton == this) {
     sVRManagerChildSingleton = nullptr;
   }
 }
 
-/*static*/
-void VRManagerChild::DeferredDestroy(RefPtr<VRManagerChild> aVRManagerChild) {
-  aVRManagerChild->Close();
-}
-
-void VRManagerChild::Destroy() {
-  // Keep ourselves alive until everything has been shut down
-  RefPtr<VRManagerChild> selfRef = this;
-
-  // The DeferredDestroyVRManager task takes ownership of
-  // the VRManagerChild and will release it when it runs.
-  MessageLoop::current()->PostTask(NewRunnableFunction(
-      "VRManagerChildDestroyRunnable", DeferredDestroy, selfRef));
-}
-
 PVRLayerChild* VRManagerChild::AllocPVRLayerChild(const uint32_t& aDisplayID,
                                                   const uint32_t& aGroup) {
   return VRLayerChild::CreateIPDLActor();
@@ -152,25 +163,34 @@ bool VRManagerChild::DeallocPVRLayerChild(PVRLayerChild* actor) {
   return VRLayerChild::DestroyIPDLActor(actor);
 }
 
-void VRManagerChild::UpdateDisplayInfo(
-    nsTArray<VRDisplayInfo>& aDisplayUpdates) {
+void VRManagerChild::UpdateDisplayInfo(const VRDisplayInfo& aDisplayInfo) {
   nsTArray<uint32_t> disconnectedDisplays;
   nsTArray<uint32_t> connectedDisplays;
 
-  nsTArray<RefPtr<VRDisplayClient>> prevDisplays;
-  prevDisplays = mDisplays;
+  const nsTArray<RefPtr<VRDisplayClient>> prevDisplays(mDisplays.Clone());
 
   // Check if any displays have been disconnected
   for (auto& display : prevDisplays) {
     bool found = false;
-    for (auto& displayUpdate : aDisplayUpdates) {
+    if (aDisplayInfo.GetDisplayID() != 0) {
       if (display->GetDisplayInfo().GetDisplayID() ==
-          displayUpdate.GetDisplayID()) {
+          aDisplayInfo.GetDisplayID()) {
         found = true;
         break;
       }
     }
     if (!found) {
+      // In order to make the current VRDisplay can continue to apply for the
+      // newest VRDisplayInfo, we need to exit presentionation before
+      // disconnecting.
+      if (display->IsPresentationGenerationCurrent()) {
+        NotifyPresentationGenerationChangedInternal(
+            display->GetDisplayInfo().GetDisplayID());
+
+        RefPtr<VRManagerChild> vm = VRManagerChild::Get();
+        vm->FireDOMVRDisplayPresentChangeEvent(
+            display->GetDisplayInfo().GetDisplayID());
+      }
       display->NotifyDisconnected();
       disconnectedDisplays.AppendElement(
           display->GetDisplayInfo().GetDisplayID());
@@ -180,30 +200,34 @@ void VRManagerChild::UpdateDisplayInfo(
   // mDisplays could be a hashed container for more scalability, but not worth
   // it now as we expect < 10 entries.
   nsTArray<RefPtr<VRDisplayClient>> displays;
-  for (VRDisplayInfo& displayUpdate : aDisplayUpdates) {
+  if (aDisplayInfo.GetDisplayID() != 0) {
     bool isNewDisplay = true;
     for (auto& display : prevDisplays) {
       const VRDisplayInfo& prevInfo = display->GetDisplayInfo();
-      if (prevInfo.GetDisplayID() == displayUpdate.GetDisplayID()) {
-        if (displayUpdate.GetIsConnected() && !prevInfo.GetIsConnected()) {
-          connectedDisplays.AppendElement(displayUpdate.GetDisplayID());
+      if (prevInfo.GetDisplayID() == aDisplayInfo.GetDisplayID()) {
+        if (aDisplayInfo.GetIsConnected() && !prevInfo.GetIsConnected()) {
+          connectedDisplays.AppendElement(aDisplayInfo.GetDisplayID());
         }
-        if (!displayUpdate.GetIsConnected() && prevInfo.GetIsConnected()) {
-          disconnectedDisplays.AppendElement(displayUpdate.GetDisplayID());
+        if (!aDisplayInfo.GetIsConnected() && prevInfo.GetIsConnected()) {
+          disconnectedDisplays.AppendElement(aDisplayInfo.GetDisplayID());
         }
-        display->UpdateDisplayInfo(displayUpdate);
+        // MOZ_KnownLive because 'prevDisplays' is guaranteed to keep it alive.
+        //
+        // This can go away once
+        // https://bugzilla.mozilla.org/show_bug.cgi?id=1620312 is fixed.
+        MOZ_KnownLive(display)->UpdateDisplayInfo(aDisplayInfo);
         displays.AppendElement(display);
         isNewDisplay = false;
         break;
       }
     }
     if (isNewDisplay) {
-      displays.AppendElement(new VRDisplayClient(displayUpdate));
-      connectedDisplays.AppendElement(displayUpdate.GetDisplayID());
+      displays.AppendElement(new VRDisplayClient(aDisplayInfo));
+      connectedDisplays.AppendElement(aDisplayInfo.GetDisplayID());
     }
   }
 
-  mDisplays = displays;
+  mDisplays = std::move(displays);
 
   // We wish to fire the events only after mDisplays is updated
   for (uint32_t displayID : disconnectedDisplays) {
@@ -213,13 +237,37 @@ void VRManagerChild::UpdateDisplayInfo(
   for (uint32_t displayID : connectedDisplays) {
     FireDOMVRDisplayConnectEvent(displayID);
   }
+}
 
-  mDisplaysInitialized = true;
+bool VRManagerChild::RuntimeSupportsVR() const {
+  return bool(mRuntimeCapabilities & VRDisplayCapabilityFlags::Cap_ImmersiveVR);
+}
+bool VRManagerChild::RuntimeSupportsAR() const {
+  return bool(mRuntimeCapabilities & VRDisplayCapabilityFlags::Cap_ImmersiveAR);
+}
+bool VRManagerChild::RuntimeSupportsInline() const {
+  return bool(mRuntimeCapabilities & VRDisplayCapabilityFlags::Cap_Inline);
+}
+
+mozilla::ipc::IPCResult VRManagerChild::RecvUpdateRuntimeCapabilities(
+    const VRDisplayCapabilityFlags& aCapabilities) {
+  mRuntimeCapabilities = aCapabilities;
+  nsContentUtils::AddScriptRunner(NewRunnableMethod<>(
+      "gfx::VRManagerChild::NotifyRuntimeCapabilitiesUpdatedInternal", this,
+      &VRManagerChild::NotifyRuntimeCapabilitiesUpdatedInternal));
+  return IPC_OK();
+}
+
+void VRManagerChild::NotifyRuntimeCapabilitiesUpdatedInternal() {
+  const nsTArray<RefPtr<VRManagerEventObserver>> listeners = mListeners.Clone();
+  for (auto& listener : listeners) {
+    listener->NotifyDetectRuntimesCompleted();
+  }
 }
 
 mozilla::ipc::IPCResult VRManagerChild::RecvUpdateDisplayInfo(
-    nsTArray<VRDisplayInfo>&& aDisplayUpdates) {
-  UpdateDisplayInfo(aDisplayUpdates);
+    const VRDisplayInfo& aDisplayInfo) {
+  UpdateDisplayInfo(aDisplayInfo);
   for (auto& windowId : mNavigatorCallbacks) {
     /** We must call NotifyVRDisplaysUpdated for every
      * window's Navigator in mNavigatorCallbacks to ensure that
@@ -239,13 +287,64 @@ mozilla::ipc::IPCResult VRManagerChild::RecvUpdateDisplayInfo(
     nav->NotifyVRDisplaysUpdated();
   }
   mNavigatorCallbacks.Clear();
+  if (mWaitingForEnumeration) {
+    nsContentUtils::AddScriptRunner(NewRunnableMethod<>(
+        "gfx::VRManagerChild::NotifyEnumerationCompletedInternal", this,
+        &VRManagerChild::NotifyEnumerationCompletedInternal));
+    mWaitingForEnumeration = false;
+  }
   return IPC_OK();
 }
 
-bool VRManagerChild::GetVRDisplays(
+mozilla::ipc::IPCResult VRManagerChild::RecvNotifyPuppetCommandBufferCompleted(
+    bool aSuccess) {
+  RefPtr<dom::Promise> promise = mRunPuppetPromise;
+  mRunPuppetPromise = nullptr;
+  if (aSuccess) {
+    promise->MaybeResolve(JS::UndefinedHandleValue);
+  } else {
+    promise->MaybeRejectWithUndefined();
+  }
+  return IPC_OK();
+}
+
+mozilla::ipc::IPCResult VRManagerChild::RecvNotifyPuppetResetComplete() {
+  nsTArray<RefPtr<dom::Promise>> promises;
+  promises.AppendElements(mResetPuppetPromises);
+  mResetPuppetPromises.Clear();
+  for (const auto& promise : promises) {
+    promise->MaybeResolve(JS::UndefinedHandleValue);
+  }
+  return IPC_OK();
+}
+
+void VRManagerChild::RunPuppet(const nsTArray<uint64_t>& aBuffer,
+                               dom::Promise* aPromise, ErrorResult& aRv) {
+  if (mRunPuppetPromise) {
+    // We only allow one puppet script to run simultaneously.
+    // The prior promise must be resolved before running a new
+    // script.
+    aRv.Throw(NS_ERROR_INVALID_ARG);
+    return;
+  }
+  if (!SendRunPuppet(aBuffer)) {
+    aRv.Throw(NS_ERROR_FAILURE);
+    return;
+  }
+  mRunPuppetPromise = aPromise;
+}
+
+void VRManagerChild::ResetPuppet(dom::Promise* aPromise, ErrorResult& aRv) {
+  if (!SendResetPuppet()) {
+    aRv.Throw(NS_ERROR_FAILURE);
+    return;
+  }
+  mResetPuppetPromises.AppendElement(aPromise);
+}
+
+void VRManagerChild::GetVRDisplays(
     nsTArray<RefPtr<VRDisplayClient>>& aDisplays) {
-  aDisplays = mDisplays;
-  return true;
+  aDisplays = mDisplays.Clone();
 }
 
 bool VRManagerChild::RefreshVRDisplaysWithCallback(uint64_t aWindowId) {
@@ -256,52 +355,33 @@ bool VRManagerChild::RefreshVRDisplaysWithCallback(uint64_t aWindowId) {
   return success;
 }
 
-void VRManagerChild::CreateVRServiceTestDisplay(const nsCString& aID,
-                                                dom::Promise* aPromise) {
-  SendCreateVRServiceTestDisplay(aID, mPromiseID);
-  mPromiseList.Put(mPromiseID, aPromise);
-  ++mPromiseID;
+bool VRManagerChild::EnumerateVRDisplays() {
+  bool success = SendRefreshDisplays();
+  if (success) {
+    mWaitingForEnumeration = true;
+  }
+  return success;
 }
 
-void VRManagerChild::CreateVRServiceTestController(const nsCString& aID,
-                                                   dom::Promise* aPromise) {
-  SendCreateVRServiceTestController(aID, mPromiseID);
-  mPromiseList.Put(mPromiseID, aPromise);
-  ++mPromiseID;
-}
+void VRManagerChild::DetectRuntimes() { Unused << SendDetectRuntimes(); }
 
 PVRLayerChild* VRManagerChild::CreateVRLayer(uint32_t aDisplayID,
-                                             nsIEventTarget* aTarget,
                                              uint32_t aGroup) {
   PVRLayerChild* vrLayerChild = AllocPVRLayerChild(aDisplayID, aGroup);
-  // Do the DOM labeling.
-  if (aTarget) {
-    SetEventTargetForActor(vrLayerChild, aTarget);
-    MOZ_ASSERT(vrLayerChild->GetActorEventTarget());
-  }
   return SendPVRLayerConstructor(vrLayerChild, aDisplayID, aGroup);
 }
 
-// XXX TODO - VRManagerChild::FrameRequest is the same as
-// Document::FrameRequest, should we consolodate these?
-struct VRManagerChild::FrameRequest {
-  FrameRequest(mozilla::dom::FrameRequestCallback& aCallback, int32_t aHandle)
-      : mCallback(&aCallback), mHandle(aHandle) {}
-
-  // Conversion operator so that we can append these to a
-  // FrameRequestCallbackList
-  operator const RefPtr<mozilla::dom::FrameRequestCallback>&() const {
-    return mCallback;
+void VRManagerChild::XRFrameRequest::Call(
+    const DOMHighResTimeStamp& aTimeStamp) {
+  if (mCallback) {
+    RefPtr<mozilla::dom::FrameRequestCallback> callback = mCallback;
+    callback->Call(aTimeStamp);
+  } else {
+    RefPtr<mozilla::dom::XRFrameRequestCallback> callback = mXRCallback;
+    RefPtr<mozilla::dom::XRFrame> frame = mXRFrame;
+    callback->Call(aTimeStamp, *frame);
   }
-
-  // Comparator operators to allow RemoveElementSorted with an
-  // integer argument on arrays of FrameRequest
-  bool operator==(int32_t aHandle) const { return mHandle == aHandle; }
-  bool operator<(int32_t aHandle) const { return mHandle < aHandle; }
-
-  RefPtr<mozilla::dom::FrameRequestCallback> mCallback;
-  int32_t mHandle;
-};
+}
 
 nsresult VRManagerChild::ScheduleFrameRequestCallback(
     mozilla::dom::FrameRequestCallback& aCallback, int32_t* aHandle) {
@@ -311,9 +391,7 @@ nsresult VRManagerChild::ScheduleFrameRequestCallback(
   }
   int32_t newHandle = ++mFrameRequestCallbackCounter;
 
-  DebugOnly<FrameRequest*> request =
-      mFrameRequestCallbacks.AppendElement(FrameRequest(aCallback, newHandle));
-  NS_ASSERTION(request, "This is supposed to be infallible!");
+  mFrameRequestCallbacks.AppendElement(XRFrameRequest(aCallback, newHandle));
 
   *aHandle = newHandle;
   return NS_OK;
@@ -324,66 +402,27 @@ void VRManagerChild::CancelFrameRequestCallback(int32_t aHandle) {
   mFrameRequestCallbacks.RemoveElementSorted(aHandle);
 }
 
-mozilla::ipc::IPCResult VRManagerChild::RecvGamepadUpdate(
-    const GamepadChangeEvent& aGamepadEvent) {
-  // VRManagerChild could be at other processes, but GamepadManager
-  // only exists at the content process or the same process
-  // in non-e10s mode.
-  MOZ_ASSERT(XRE_IsContentProcess() || IsSameProcess());
-
-  RefPtr<GamepadManager> gamepadManager(GamepadManager::GetService());
-  if (gamepadManager) {
-    gamepadManager->Update(aGamepadEvent);
-  }
-
-  return IPC_OK();
-}
-
-mozilla::ipc::IPCResult VRManagerChild::RecvReplyCreateVRServiceTestDisplay(
-    const nsCString& aID, const uint32_t& aPromiseID,
-    const uint32_t& aDeviceID) {
-  RefPtr<dom::Promise> p;
-  if (!mPromiseList.Get(aPromiseID, getter_AddRefs(p))) {
-    MOZ_CRASH("We should always have a promise.");
-  }
-
-  // We only allow one VRMockDisplay in VR tests.
-  if (!mVRMockDisplay) {
-    mVRMockDisplay = new VRMockDisplay(aID, aDeviceID);
-  }
-  p->MaybeResolve(mVRMockDisplay);
-  mPromiseList.Remove(aPromiseID);
-  return IPC_OK();
-}
-
-mozilla::ipc::IPCResult VRManagerChild::RecvReplyCreateVRServiceTestController(
-    const nsCString& aID, const uint32_t& aPromiseID,
-    const uint32_t& aDeviceID) {
-  RefPtr<dom::Promise> p;
-  if (!mPromiseList.Get(aPromiseID, getter_AddRefs(p))) {
-    MOZ_CRASH("We should always have a promise.");
-  }
-
-  if (aDeviceID == 0) {
-    // A value of 0 indicates that the controller could not
-    // be created.  Most likely due to having no VR display
-    // to associate it with.
-    p->MaybeRejectWithUndefined();
-  } else {
-    p->MaybeResolve(new VRMockController(aID, aDeviceID));
-  }
-  mPromiseList.Remove(aPromiseID);
-  return IPC_OK();
-}
-
 void VRManagerChild::RunFrameRequestCallbacks() {
-  AUTO_PROFILER_TRACING("VR", "RunFrameRequestCallbacks", GRAPHICS);
+  AUTO_PROFILER_TRACING_MARKER("VR", "RunFrameRequestCallbacks", GRAPHICS);
 
   TimeStamp nowTime = TimeStamp::Now();
   mozilla::TimeDuration duration = nowTime - mStartTimeStamp;
   DOMHighResTimeStamp timeStamp = duration.ToMilliseconds();
 
-  nsTArray<FrameRequest> callbacks;
+  if (!sMostRecentFrameEnd.IsNull()) {
+    TimeDuration frameInterval = nowTime - sMostRecentFrameEnd;
+    if (sAverageFrameInterval.IsZero()) {
+      sAverageFrameInterval = frameInterval;
+    } else {
+      // Calculate the average interval between frame end and next frame start.
+      // Apply some smoothing to make it more stable.
+      const double smooth = 0.9;
+      sAverageFrameInterval = sAverageFrameInterval.MultDouble(smooth) +
+                              frameInterval.MultDouble(1.0 - smooth);
+    }
+  }
+
+  nsTArray<XRFrameRequest> callbacks;
   callbacks.AppendElements(mFrameRequestCallbacks);
   mFrameRequestCallbacks.Clear();
   for (auto& callback : callbacks) {
@@ -391,6 +430,10 @@ void VRManagerChild::RunFrameRequestCallbacks() {
     // mCallback and there's nothing that can drop that ref until we return.
     MOZ_KnownLive(callback.mCallback)->Call(timeStamp);
   }
+
+  if (IsPresenting()) {
+    sMostRecentFrameEnd = TimeStamp::Now();
+  }
 }
 
 void VRManagerChild::NotifyPresentationGenerationChanged(uint32_t aDisplayID) {
@@ -428,13 +471,16 @@ void VRManagerChild::FireDOMVRDisplayPresentChangeEvent(uint32_t aDisplayID) {
   nsContentUtils::AddScriptRunner(NewRunnableMethod<uint32_t>(
       "gfx::VRManagerChild::FireDOMVRDisplayPresentChangeEventInternal", this,
       &VRManagerChild::FireDOMVRDisplayPresentChangeEventInternal, aDisplayID));
+
+  if (!IsPresenting()) {
+    sMostRecentFrameEnd = TimeStamp();
+    sAverageFrameInterval = 0;
+  }
 }
 
 void VRManagerChild::FireDOMVRDisplayMountedEventInternal(uint32_t aDisplayID) {
   // Iterate over a copy of mListeners, as dispatched events may modify it.
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
+  for (auto& listener : mListeners.Clone()) {
     listener->NotifyVRDisplayMounted(aDisplayID);
   }
 }
@@ -442,18 +488,14 @@ void VRManagerChild::FireDOMVRDisplayMountedEventInternal(uint32_t aDisplayID) {
 void VRManagerChild::FireDOMVRDisplayUnmountedEventInternal(
     uint32_t aDisplayID) {
   // Iterate over a copy of mListeners, as dispatched events may modify it.
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
+  for (auto& listener : mListeners.Clone()) {
     listener->NotifyVRDisplayUnmounted(aDisplayID);
   }
 }
 
 void VRManagerChild::FireDOMVRDisplayConnectEventInternal(uint32_t aDisplayID) {
   // Iterate over a copy of mListeners, as dispatched events may modify it.
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
+  for (auto& listener : mListeners.Clone()) {
     listener->NotifyVRDisplayConnect(aDisplayID);
   }
 }
@@ -461,9 +503,7 @@ void VRManagerChild::FireDOMVRDisplayConnectEventInternal(uint32_t aDisplayID) {
 void VRManagerChild::FireDOMVRDisplayDisconnectEventInternal(
     uint32_t aDisplayID) {
   // Iterate over a copy of mListeners, as dispatched events may modify it.
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
+  for (auto& listener : mListeners.Clone()) {
     listener->NotifyVRDisplayDisconnect(aDisplayID);
   }
 }
@@ -471,38 +511,42 @@ void VRManagerChild::FireDOMVRDisplayDisconnectEventInternal(
 void VRManagerChild::FireDOMVRDisplayPresentChangeEventInternal(
     uint32_t aDisplayID) {
   // Iterate over a copy of mListeners, as dispatched events may modify it.
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
-    listener->NotifyVRDisplayPresentChange(aDisplayID);
+  for (auto& listener : mListeners.Clone()) {
+    // MOZ_KnownLive because 'listeners' is guaranteed to keep it alive.
+    //
+    // This can go away once
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=1620312 is fixed.
+    MOZ_KnownLive(listener)->NotifyVRDisplayPresentChange(aDisplayID);
   }
 }
 
 void VRManagerChild::FireDOMVRDisplayConnectEventsForLoadInternal(
-    uint32_t aDisplayID, dom::VREventObserver* aObserver) {
+    uint32_t aDisplayID, VRManagerEventObserver* aObserver) {
   aObserver->NotifyVRDisplayConnect(aDisplayID);
 }
 
 void VRManagerChild::NotifyPresentationGenerationChangedInternal(
     uint32_t aDisplayID) {
-  nsTArray<RefPtr<dom::VREventObserver>> listeners;
-  listeners = mListeners;
-  for (auto& listener : listeners) {
+  for (auto& listener : mListeners.Clone()) {
     listener->NotifyPresentationGenerationChanged(aDisplayID);
   }
 }
 
+void VRManagerChild::NotifyEnumerationCompletedInternal() {
+  for (auto& listener : mListeners.Clone()) {
+    listener->NotifyEnumerationCompleted();
+  }
+}
+
 void VRManagerChild::FireDOMVRDisplayConnectEventsForLoad(
-    dom::VREventObserver* aObserver) {
+    VRManagerEventObserver* aObserver) {
   // We need to fire the VRDisplayConnect event when a page is loaded
   // for each VR Display that has already been enumerated
-  nsTArray<RefPtr<VRDisplayClient>> displays;
-  displays = mDisplays;
-  for (auto& display : displays) {
+  for (const auto& display : mDisplays.Clone()) {
     const VRDisplayInfo& info = display->GetDisplayInfo();
     if (info.GetIsConnected()) {
       nsContentUtils::AddScriptRunner(NewRunnableMethod<
-                                      uint32_t, RefPtr<dom::VREventObserver>>(
+                                      uint32_t, RefPtr<VRManagerEventObserver>>(
           "gfx::VRManagerChild::FireDOMVRDisplayConnectEventsForLoadInternal",
           this, &VRManagerChild::FireDOMVRDisplayConnectEventsForLoadInternal,
           info.GetDisplayID(), aObserver));
@@ -510,7 +554,7 @@ void VRManagerChild::FireDOMVRDisplayConnectEventsForLoad(
   }
 }
 
-void VRManagerChild::AddListener(dom::VREventObserver* aObserver) {
+void VRManagerChild::AddListener(VRManagerEventObserver* aObserver) {
   MOZ_ASSERT(aObserver);
 
   if (mListeners.IndexOf(aObserver) != kNoIndex) {
@@ -523,7 +567,7 @@ void VRManagerChild::AddListener(dom::VREventObserver* aObserver) {
   }
 }
 
-void VRManagerChild::RemoveListener(dom::VREventObserver* aObserver) {
+void VRManagerChild::RemoveListener(VRManagerEventObserver* aObserver) {
   MOZ_ASSERT(aObserver);
 
   mListeners.RemoveElement(aObserver);
@@ -545,13 +589,22 @@ void VRManagerChild::StopActivity() {
   Unused << SendStopActivity();
 }
 
-void VRManagerChild::HandleFatalError(const char* aMsg) const {
+void VRManagerChild::HandleFatalError(const char* aMsg) {
   dom::ContentChild::FatalErrorIfNotUsingGPUProcess(aMsg, OtherPid());
 }
 
 void VRManagerChild::AddPromise(const uint32_t& aID, dom::Promise* aPromise) {
-  MOZ_ASSERT(!mGamepadPromiseList.Get(aID, nullptr));
-  mGamepadPromiseList.Put(aID, aPromise);
+  MOZ_ASSERT(!mGamepadPromiseList.Contains(aID));
+  mGamepadPromiseList.InsertOrUpdate(aID, RefPtr{aPromise});
+}
+
+gfx::VRAPIMode VRManagerChild::GetVRAPIMode(uint32_t aDisplayID) const {
+  for (auto& display : mDisplays) {
+    if (display->GetDisplayInfo().GetDisplayID() == aDisplayID) {
+      return display->GetXRAPIMode();
+    }
+  }
+  return VRAPIMode::WebXR;
 }
 
 mozilla::ipc::IPCResult VRManagerChild::RecvReplyGamepadVibrateHaptic(
@@ -571,18 +624,5 @@ mozilla::ipc::IPCResult VRManagerChild::RecvReplyGamepadVibrateHaptic(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult VRManagerChild::RecvDispatchSubmitFrameResult(
-    const uint32_t& aDisplayID, const VRSubmitFrameResultInfo& aResult) {
-  nsTArray<RefPtr<VRDisplayClient>> displays;
-  displays = mDisplays;
-  for (auto& display : displays) {
-    if (display->GetDisplayInfo().GetDisplayID() == aDisplayID) {
-      display->UpdateSubmitFrameResult(aResult);
-    }
-  }
-
-  return IPC_OK();
-}
-
 }  // namespace gfx
 }  // namespace mozilla
