@@ -198,6 +198,8 @@ task_description_schema = Schema(
         },
         # Override the default priority for the project
         Optional("priority"): str,
+        # Override the default 5 retries
+        Optional("retries"): int,
     }
 )
 
@@ -521,19 +523,11 @@ def build_docker_worker_payload(config, task, task_def):
 
     if "artifacts" in worker:
         artifacts = {}
-        expires_policy = get_expiration(
-            config, task.get("expiration-policy", "default")
-        )
-        now = datetime.datetime.utcnow()
-        task_exp = task_def["expires"]["relative-datestamp"]
-        task_exp_from_now = fromNow(task_exp)
         for artifact in worker["artifacts"]:
-            art_exp = artifact.get("expires-after", expires_policy)
-            expires = art_exp if fromNow(art_exp, now) < task_exp_from_now else task_exp
             artifacts[artifact["name"]] = {
                 "path": artifact["path"],
                 "type": artifact["type"],
-                "expires": {"relative-datestamp": expires},
+                "expires": {"relative-datestamp": artifact["expires-after"]},
             }
         payload["artifacts"] = artifacts
 
@@ -755,18 +749,11 @@ def build_generic_worker_payload(config, task, task_def):
 
     artifacts = []
 
-    expires_policy = get_expiration(config, task.get("expiration-policy", "default"))
-    now = datetime.datetime.utcnow()
-    task_exp = task_def["expires"]["relative-datestamp"]
-    task_exp_from_now = fromNow(task_exp)
     for artifact in worker.get("artifacts", []):
-        art_exp = artifact.get("expires-after", expires_policy)
-        task_exp = task_def["expires"]["relative-datestamp"]
-        expires = art_exp if fromNow(art_exp, now) < task_exp_from_now else task_exp
         a = {
             "path": artifact["path"],
             "type": artifact["type"],
-            "expires": {"relative-datestamp": expires},
+            "expires": {"relative-datestamp": artifact["expires-after"]},
         }
         if "name" in artifact:
             a["name"] = artifact["name"]
@@ -1322,20 +1309,23 @@ def build_push_addons_payload(config, task, task_def):
         Optional("push"): bool,
         Optional("source-repo"): str,
         Optional("ssh-user"): str,
-        Optional("l10n-bump-info"): {
-            Required("name"): str,
-            Required("path"): str,
-            Required("version-path"): str,
-            Optional("l10n-repo-url"): str,
-            Optional("ignore-config"): object,
-            Required("platform-configs"): [
-                {
-                    Required("platforms"): [str],
-                    Required("path"): str,
-                    Optional("format"): str,
-                }
-            ],
-        },
+        Optional("l10n-bump-info"): [
+            {
+                Required("name"): str,
+                Required("path"): str,
+                Required("version-path"): str,
+                Optional("l10n-repo-url"): str,
+                Optional("l10n-repo-target-branch"): str,
+                Optional("ignore-config"): object,
+                Required("platform-configs"): [
+                    {
+                        Required("platforms"): [str],
+                        Required("path"): str,
+                        Optional("format"): str,
+                    }
+                ],
+            }
+        ],
         Optional("merge-info"): object,
     },
 )
@@ -1378,11 +1368,26 @@ def build_treescript_payload(config, task, task_def):
         actions.append("version_bump")
 
     if worker.get("l10n-bump-info"):
-        l10n_bump_info = {}
-        for k, v in worker["l10n-bump-info"].items():
-            l10n_bump_info[k.replace("-", "_")] = worker["l10n-bump-info"][k]
-        task_def["payload"]["l10n_bump_info"] = [l10n_bump_info]
-        actions.append("l10n_bump")
+        l10n_bump_info = []
+        l10n_repo_urls = set()
+        for lbi in worker["l10n-bump-info"]:
+            new_lbi = {}
+            if "l10n-repo-url" in lbi:
+                l10n_repo_urls.add(lbi["l10n-repo-url"])
+            for k, v in lbi.items():
+                new_lbi[k.replace("-", "_")] = lbi[k]
+            l10n_bump_info.append(new_lbi)
+
+        task_def["payload"]["l10n_bump_info"] = l10n_bump_info
+        if len(l10n_repo_urls) > 1:
+            raise Exception(
+                "Must use the same l10n-repo-url for all files in the same task!"
+            )
+        elif len(l10n_repo_urls) == 1:
+            if "github.com" in l10n_repo_urls.pop():
+                actions.append("l10n_bump_github")
+            else:
+                actions.append("l10n_bump")
 
     if worker.get("merge-info"):
         merge_info = {
@@ -1419,6 +1424,128 @@ def build_treescript_payload(config, task, task_def):
         task_def["payload"]["ssh_user"] = worker["ssh-user"]
 
 
+@payload_builder(
+    "landoscript",
+    schema={
+        Required("lando-repo"): str,
+        Optional("hg-repo-url"): str,
+        Optional("ignore-closed-tree"): bool,
+        Optional("dontbuild"): bool,
+        Optional("tags"): [Any("buildN", "release", None)],
+        Optional("force-dry-run"): bool,
+        Optional("l10n-bump-info"): [
+            {
+                Required("name"): str,
+                Required("path"): str,
+                Optional("l10n-repo-url"): str,
+                Optional("l10n-repo-target-branch"): str,
+                Optional("ignore-config"): object,
+                Required("platform-configs"): [
+                    {
+                        Required("platforms"): [str],
+                        Required("path"): str,
+                        Optional("format"): str,
+                    }
+                ],
+            }
+        ],
+        Optional("bump-files"): [str],
+        Optional("merge-info"): object,
+    },
+)
+def build_landoscript_payload(config, task, task_def):
+    worker = task["worker"]
+    release_config = get_release_config(config)
+    task_def["payload"] = {"actions": [], "lando_repo": worker["lando-repo"]}
+    actions = task_def["payload"]["actions"]
+
+    if worker.get("ignore-closed-tree") is not None:
+        task_def["payload"]["ignore_closed_tree"] = worker["ignore-closed-tree"]
+
+    if worker.get("dontbuild"):
+        task_def["payload"]["dontbuild"] = True
+
+    if worker.get("force-dry-run"):
+        task_def["payload"]["dry_run"] = True
+
+    if worker.get("l10n-bump-info"):
+        l10n_bump_info = []
+        l10n_repo_urls = set()
+        for lbi in worker["l10n-bump-info"]:
+            new_lbi = {}
+            if "l10n-repo-url" in lbi:
+                l10n_repo_urls.add(lbi["l10n-repo-url"])
+            for k, v in lbi.items():
+                new_lbi[k.replace("-", "_")] = lbi[k]
+            l10n_bump_info.append(new_lbi)
+
+        task_def["payload"]["l10n_bump_info"] = l10n_bump_info
+        if len(l10n_repo_urls) > 1:
+            raise Exception(
+                "Must use the same l10n-repo-url for all files in the same task!"
+            )
+        elif len(l10n_repo_urls) == 1:
+            actions.append("l10n_bump")
+
+    if worker.get("tags"):
+        tag_names = []
+        product = task["shipping-product"].upper()
+        version = release_config["version"].replace(".", "_")
+        buildnum = release_config["build_number"]
+        if "buildN" in worker["tags"]:
+            tag_names.extend(
+                [
+                    f"{product}_{version}_BUILD{buildnum}",
+                ]
+            )
+        if "release" in worker["tags"]:
+            tag_names.extend([f"{product}_{version}_RELEASE"])
+        tag_info = {
+            "tags": tag_names,
+            "hg_repo_url": worker["hg-repo-url"],
+            "revision": config.params[
+                "{}head_rev".format(worker.get("repo-param-prefix", ""))
+            ],
+        }
+        task_def["payload"]["tag_info"] = tag_info
+        actions.append("tag")
+
+    if worker.get("bump-files"):
+        bump_info = {}
+        bump_info["next_version"] = release_config["next_version"]
+        bump_info["files"] = worker["bump-files"]
+        task_def["payload"]["version_bump_info"] = bump_info
+        actions.append("version_bump")
+
+    if worker.get("merge-info"):
+        merge_info = {
+            merge_param_name.replace("-", "_"): merge_param_value
+            for merge_param_name, merge_param_value in worker["merge-info"].items()
+            if merge_param_name != "version-files"
+        }
+        merge_info["version_files"] = [
+            {
+                file_param_name.replace("-", "_"): file_param_value
+                for file_param_name, file_param_value in file_entry.items()
+            }
+            for file_entry in worker["merge-info"]["version-files"]
+        ]
+        # hack alert: co-opt the l10n_bump_info into the merge_info section
+        # this should be cleaned up to avoid l10n_bump_info ever existing
+        # in the payload
+        if task_def["payload"].get("l10n_bump_info"):
+            actions.remove("l10n_bump")
+            merge_info["l10n_bump_info"] = task_def["payload"].pop("l10n_bump_info")
+
+        task_def["payload"]["merge_info"] = merge_info
+        actions.append("merge_day")
+
+    scopes = set(task_def.get("scopes", []))
+    scopes.add(f"project:releng:lando:repo:{worker['lando-repo']}")
+    scopes.update([f"project:releng:lando:action:{action}" for action in actions])
+    task_def["scopes"] = sorted(scopes)
+
+
 @payload_builder(
     "invalid",
     schema={
@@ -1847,13 +1974,31 @@ def set_task_and_artifact_expiry(config, jobs):
     These values are read from ci/config.yml
     """
     now = datetime.datetime.utcnow()
+    # We don't want any configuration leading to anything with an expiry longer
+    # than 28 days on try.
+    cap = "28 days" if is_try(config.params) else None
+    cap_from_now = fromNow(cap, now) if cap else None
+    if cap:
+        for policy, expires in config.graph_config["expiration-policy"]["by-project"][
+            "try"
+        ].items():
+            if fromNow(expires, now) > cap_from_now:
+                raise Exception(
+                    f'expiration-policy "{policy}" is larger than {cap} '
+                    f'for {config.params["project"]}'
+                )
     for job in jobs:
         expires = get_expiration(config, job.get("expiration-policy", "default"))
         job_expiry = job.setdefault("expires-after", expires)
-        job_expiry_from_now = fromNow(job_expiry)
+        job_expiry_from_now = fromNow(job_expiry, now)
+        if cap and job_expiry_from_now > cap_from_now:
+            job_expiry, job_expiry_from_now = cap, cap_from_now
+        # If the task has no explicit expiration-policy, but has an expires-after,
+        # we use that as the default artifact expiry.
+        artifact_expires = expires if "expiration-policy" in job else job_expiry
 
         for artifact in job["worker"].get("artifacts", ()):
-            artifact_expiry = artifact.setdefault("expires-after", expires)
+            artifact_expiry = artifact.setdefault("expires-after", artifact_expires)
 
             # By using > instead of >=, there's a chance of mismatch
             #   where the artifact expires sooner than the task.
@@ -1974,6 +2119,8 @@ def build_task(config, tasks):
 
         if task.get("requires", None):
             task_def["requires"] = task["requires"]
+        if task.get("retries") is not None:
+            task_def["retries"] = task["retries"]
 
         if task_th:
             # link back to treeherder in description
